#include <harkio2/hark/HarkFile.hpp>
#include <harkio2/hark/DataSet.hpp>

namespace harkio2
{
namespace hark
{

boost::shared_ptr<harkio2::hark::DataSet> HarkFile::GetDataSet(size_t index) const throw (std::out_of_range)
{
    return dataSets.at(index);
}

boost::shared_ptr<harkio2::hark::DataSet> HarkFile::GetDataSet(const std::string& id) const throw ()
{
    for (size_t i = 0; i < dataSets.size(); ++i)
    {
        if (id == dataSets[i]->GetId())
        {
            return dataSets[i];
        }
    }
    return boost::shared_ptr<harkio2::hark::DataSet>();
}

std::vector<boost::shared_ptr<harkio2::hark::DataSet> > HarkFile::SearchDataSets(const std::string& id) const throw ()
{
    std::vector<boost::shared_ptr<harkio2::hark::DataSet> > result;

    for (size_t i = 0; i < dataSets.size(); ++i)
    {
        if (id == dataSets[i]->GetId())
        {
            result.push_back(dataSets[i]);
        }
    }

    return result;
}

void HarkFile::AddDataSet(boost::shared_ptr<harkio2::hark::DataSet> dataSet) throw ()
{
    dataSets.push_back(dataSet);
}

void HarkFile::RemoveDataSet(size_t index) throw (std::out_of_range)
{
    if (index >= dataSets.size())
    {
        throw std::out_of_range("index is out of range");
    }

    std::vector<boost::shared_ptr<harkio2::hark::DataSet> >::iterator it = dataSets.begin() + index;
    dataSets.erase(it);
}

void HarkFile::RemoveDataSet(boost::shared_ptr<harkio2::hark::DataSet> dataSet) throw ()
{
    std::vector<boost::shared_ptr<harkio2::hark::DataSet> >::iterator it;
    for (it = dataSets.begin(); it != dataSets.end(); ++it)
    {
        if (dataSet.get() == it->get())
        {
            dataSets.erase(it);
            break;
        }
    }
}

void HarkFile::RemoveDataSet(const std::string& id) throw ()
{
    std::vector<boost::shared_ptr<harkio2::hark::DataSet> >::iterator it;
    for (it = dataSets.begin(); it != dataSets.end(); ++it)
    {
        if (id == (*it)->GetId())
        {
            dataSets.erase(it);
            break;
        }
    }
}

size_t HarkFile::GetDataSetsSize() const throw ()
{
    return dataSets.size();
}

bool IsValidHarkFile(const harkio2::hark::HarkFile& harkFile) throw ()
{
    typedef boost::shared_ptr<harkio2::hark::DataSet> DataSetPtr;
    if (harkFile.GetDataSetsSize() == 0)
    {
        return true;
    }
    else if (harkFile.GetDataSet("hgtf::Fmt") != 0)
    {
        DataSetPtr m2pg = harkFile.GetDataSet("hgtf::M2PG");
        DataSetPtr gtf = harkFile.GetDataSet("hgtf::GTF");
        DataSetPtr sm = harkFile.GetDataSet("hgtf::SM");

        if ((m2pg && !gtf && !sm) || (!m2pg && gtf && !sm) || (!m2pg && !gtf && sm))
        {
            DataSetPtr spec = harkFile.GetDataSet("hgtf::Spec");
            if ((harkFile.GetDataSetsSize() == 3 && spec) ||
                    (harkFile.GetDataSetsSize() == 2 && !spec))
            {
                return true;
            }
        }
    }
    else if (harkFile.GetDataSet("gfbc::Fmt") != 0)
    {
    }

    return false;

}

}
}

